/*******************************************************************************
  MPLAB Harmony Application Source File
  
  Company:
    Microchip Technology Inc.
  
  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It 
    implements the logic of the application's state machine and it may call 
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013-2014 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
// DOM-IGNORE-END


// *****************************************************************************
// *****************************************************************************
// Section: Included Files 
// *****************************************************************************
// *****************************************************************************

#include "app.h"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.
    
    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;
// This is the string to write to the slave device.  Be aware that the double quotes adds a null byte at the end of the string.
// So, writing "Hello World!" actually transmits 13 bytes.
uint8_t appWriteString[] = "Hello World";
const I2C_SLAVE_ADDRESS_VALUE appSlaveAddress = 0x40;


static uint8_t usartBQTxData[] = "Hello World\r\n";
static enum 
{
    USART_BQ_INIT,
    USART_BQ_WORKING,
    USART_BQ_DONE,
} usartBQState;

static char appMsgToClient[] = "Hello Client!\n\r";
static uint8_t appMsgFromClient[80];

static TCPIP_NET_HANDLE    	app_netH;
static SYS_STATUS          	app_tcpipStat;
static int                 	app_nNets;

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

void (DRV_I2C_BUFFER_EVENT event, void * context)
{
        switch (event)
    {
        case DRV_I2C_BUFFER_SLAVE_READ_REQUESTED:
            appData.I2CBufferHandle = DRV_I2C_Receive (   appData.handleI2C0, 
                                                                appSlaveAddress,
                                                                appReadBuffer,
                                                                sizeof(appReadBuffer),
                                                                NULL);
            break;
        case DRV_I2C_BUFFER_SLAVE_WRITE_REQUESTED:                        
            appData.I2CBufferHandle = DRV_I2C_Transmit (   appData.handleI2C0, 
                                                                appSlaveAddress,
                                                                appReadBuffer,
                                                                sizeof(appReadBuffer),
                                                                NULL);
            Nop();
            break;
        default:
            break;
    }
}


/* Application's Timer Callback Function */
static void TimerCallback (  uintptr_t context, uint32_t alarmCount )
{
}

/* TODO:  Add any necessary callback functions.
*/

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************


/* Application's i2c Setup Function */
static void I2C_Setup( void )
{
    appData.i2cStates = APP_I2C_START;
}
/******************************************************************************
  Function:
    static void APP_I2C_Task (void)
    
   Remarks:
    Allows a polled state machine to manage i2c testing.

*/
static void APP_I2C_Task (void)
{		
	switch(appData.i2cStates)
	{
		default:
		
		case APP_I2C_START:
        {
            // Switch to the Transmit State.
            appData.i2cStates = APP_I2C_TRANSMIT;
        
			break;
		}	
		case APP_I2C_TRANSMIT:
        {
            // If no transmission has occured yet...
            if(appData.I2CBufferHandle == NULL)
            {
                appData.I2CBufferHandle = DRV_I2C_Transmit ( appData.handleI2C0,
                                                        appSlaveAddress,
                                                        appWriteString,
                                                        sizeof(appWriteString), 
                                                        NULL);
            } else {
                appData.I2CBufferEvent = DRV_I2C_TransferStatusGet ( appData.handleI2C0,
                         appData.I2CBufferHandle );
                if(appData.I2CBufferEvent == DRV_I2C_BUFFER_EVENT_COMPLETE) 
                    {
                        appData.i2cStates = APP_I2C_DONE;
                    }
                if(appData.I2CBufferEvent == DRV_I2C_BUFFER_EVENT_ERROR)
                    {
                        appData.i2cStates = APP_I2C_ERROR;
                    }
            }

            break;
		}
		case APP_I2C_DONE:
		{
			break;
        }
        case APP_I2C_ERROR:
		{
            break;
        }

    }
}

/* Application's Timer Setup Function */
static void TimerSetup( void )
{
    DRV_TMR_AlarmRegister(
        appData.handleTimer1, 
        APP_TMR_DRV_PERIOD, 
        APP_TMR_DRV_IS_PERIODIC,
        (uintptr_t)NULL, 
        TimerCallback);
    DRV_TMR_Start(appData.handleTimer1);
}
/******************************************************************************
  Function:
    void BufferQueueEventHandler ( DRV_USART_BUFFER_EVENT,
                DRV_USART_BUFFER_HANDLE,  uintptr_t );

   Remarks:
    This routine is callback function for the USART buffer queue events. Driver
    uses the application Tx or Rx buffer reference along with the client handle
    to notify the buffer events to the application. This call back function 
    to be registerd with the driver using DRV_USART_BufferAddRead(); 
    or DRV_USART_BufferAddWrite();
 */

static void BufferQueueEventHandler(DRV_USART_BUFFER_EVENT  bufEvent,
                                    DRV_USART_BUFFER_HANDLE bufHandle,
                                    uintptr_t               context )
{
    int i;
 
    /*  Does bufHandle identify an RX buffer?  */
    if (appData.usartBQRxBufferHandle == bufHandle)
    {
        /* Transfer buffer to the Application by invalidating the buffer handle */
        appData.usartBQRxBufferHandle = DRV_USART_BUFFER_HANDLE_INVALID;

        /* Zero the number of characters written in preparation for transmitting them */
        appData.usartBQRxWritten = 0;
          
        /* Set the number of characters read */              
        if (bufEvent == DRV_USART_BUFFER_EVENT_COMPLETE)
        {
            /* Successful read */
            appData.usartBQRxRead = APP_DRV_USART_BQ_RX_SIZE;
        }
        else
        {
            /* Read was not successful */
            appData.usartBQRxRead = 0;
        }
        
        return;
    }

    /*  Does the buffer handle identify a TX buffer?  */
    if (appData.usartBQTxBufferHandle == bufHandle)
    {
        /* Transfer buffer to the Application by invalidating the buffer handle */
        appData.usartBQTxBufferHandle = DRV_USART_BUFFER_HANDLE_INVALID;
        appData.usartBQTxWritten = sizeof(usartBQTxData);
        return;
    }
}

/******************************************************************************
  Function:
    static void USART_Task (void)
    
   Remarks:
    Feeds the USART transmitter.
*/
static void USART_Task (void)
{
    switch (usartBQState)
    {
        default:
        case USART_BQ_INIT:
        {
            appData.usartBQTxWritten = 0;
            appData.usartBQRxRead    = 0;
            appData.usartBQRxWritten = 0;

            usartBQState = USART_BQ_WORKING;
            break;
        }

        case USART_BQ_WORKING:
        {
            /* Check to see if the Application owns the TX buffer */
            if (appData.usartBQTxBufferHandle == DRV_USART_BUFFER_HANDLE_INVALID)
            {
                /* If we have data to transmit, then queue the buffer */
                if (appData.usartBQTxWritten == 0)
                {
                    DRV_USART_BufferAddWrite(appData.handleUSART0,
                            (DRV_USART_BUFFER_HANDLE * const)&appData.usartBQTxBufferHandle,
                            &usartBQTxData[0], sizeof(usartBQTxData));
                }
            }

            /* Has the RX buffer been read by the USART driver and transferred to the application by the BufferQueueEventHandler function? */
            if (appData.usartBQRxBufferHandle == DRV_USART_BUFFER_HANDLE_INVALID)
            {
                /* Process this buffer */
                if (appData.usartBQRxRead == 0)
                {
                    DRV_USART_BufferAddRead(appData.handleUSART0,
                        (DRV_USART_BUFFER_HANDLE * const)&appData.usartBQRxBufferHandle,
                        &appData.usartBQRxData[0], APP_DRV_USART_BQ_RX_SIZE);
                }
            }

            if ((appData.usartBQTxWritten == sizeof(usartBQTxData)) && (appData.usartBQRxRead == APP_DRV_USART_BQ_RX_SIZE))
            {
                usartBQState = USART_BQ_DONE;
            }
            break;
        }

        case USART_BQ_DONE:
        {
            break;
        }
    }
}
/******************************************************************************
  Function:
    static void TCP_Server_TXRX_Task (void)
    
   Remarks:
    Feeds the USB write function. 
*/
static void TCP_Server_TXRX_Task (void)
{
	static IPV4_ADDR    		dwLastIP[2] = { {-1}, {-1} };
	static IPV4_ADDR           	ipAddr;
	int                 		i;

	switch (appData.txrxTaskState)
	{
        case APP_TCPIP_WAIT_FOR_IP:
        {
            app_nNets = TCPIP_STACK_NumberOfNetworksGet();

            for (i = 0; i < app_nNets; i++)
            {
                app_netH = TCPIP_STACK_IndexToNet(i);
                ipAddr.Val = TCPIP_STACK_NetAddress(app_netH);
                if (TCPIP_STACK_NetIsReady(app_netH))
                {
                    appData.txrxTaskState = APP_TCPIP_OPENING_SERVER;
                }
            }
            break;
        }
        case APP_TCPIP_OPENING_SERVER:
        {
            appData.socket = TCPIP_TCP_ServerOpen(IP_ADDRESS_TYPE_IPV4, appData.port, 0);
            if (appData.socket == INVALID_SOCKET)
            {
                break;
            }
            appData.txrxTaskState = APP_TCPIP_WAIT_FOR_CONNECTION;;
        }
        break;

        case APP_TCPIP_WAIT_FOR_CONNECTION:
        {
            if (!TCPIP_TCP_IsConnected(appData.socket))
            {
                break;
            }
            else
            {
                // We got a connection
				TCPIP_TCP_ArrayPut(appData.socket, appMsgToClient, sizeof(appMsgToClient));
                appData.txrxTaskState = APP_TCPIP_WAIT_FOR_RESPONSE;
            }
        }
        break;

        case APP_TCPIP_WAIT_FOR_RESPONSE:
        {
            if (!TCPIP_TCP_IsConnected(appData.socket))
            {
                appData.txrxTaskState = APP_TCPIP_WAIT_FOR_IP;
                break;
            }
            if (TCPIP_TCP_GetIsReady(appData.socket))
            {
                TCPIP_TCP_ArrayGet(appData.socket, appMsgFromClient, sizeof(appMsgFromClient) - 1);
            }
        }
        break;

        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}

/* TODO:  Add any necessary local functions.
*/


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;

    appData.handleI2C0 = DRV_HANDLE_INVALID;
    appData.handleTimer1 = DRV_HANDLE_INVALID;
    appData.handleUSART0 = DRV_HANDLE_INVALID;
    
    /* Initialize the receive buffer as owned by the Application with no data */
    appData.usartBQRxBufferHandle = DRV_USART_BUFFER_HANDLE_INVALID;

    appData.usartBQTxBufferHandle = DRV_USART_BUFFER_HANDLE_INVALID;
	appData.port = 53510;
    
    /* TODO: Initialize your application's state machine and other
     * parameters.
     */
}


/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Tasks ( void )
{
    int i;

    /* Check the application's current state. */
    switch ( appData.state )
    {
        /* Application's initial state. */
        case APP_STATE_INIT:
        {
            bool appInitialized = true;
       
            if (appData.handleI2C0 == DRV_HANDLE_INVALID)
            {
                appData.handleI2C0 = DRV_I2C_Open(APP_DRV_I2C_INDEX, DRV_IO_INTENT_EXCLUSIVE);
                appInitialized &= ( DRV_HANDLE_INVALID != appData.handleI2C0 );
            }
            if (appData.handleTimer1 == DRV_HANDLE_INVALID)
            {
                appData.handleTimer1 = DRV_TMR_Open(APP_TMR_DRV, DRV_IO_INTENT_EXCLUSIVE);
                appInitialized &= ( DRV_HANDLE_INVALID != appData.handleTimer1 );
            }
            /* Open the USART driver if it has not been previously opened */
            if (appData.handleUSART0 == DRV_HANDLE_INVALID)
            {
                appData.handleUSART0 = DRV_USART_Open(APP_DRV_USART, DRV_IO_INTENT_READWRITE|DRV_IO_INTENT_NONBLOCKING);
                appInitialized &= ( DRV_HANDLE_INVALID != appData.handleUSART0 );
            }
            app_tcpipStat = TCPIP_STACK_Status(sysObj.tcpip);
            if(app_tcpipStat < 0)
            {   // some error occurred
                appData.state = APP_STATE_ERROR;
				appInitialized = false;
            }
            else if(app_tcpipStat == SYS_STATUS_READY)
            {
                // now that the stack is ready we can check the
                // available interfaces
                app_nNets = TCPIP_STACK_NumberOfNetworksGet();
                for(i = 0; i < app_nNets; i++)
                {
                    app_netH = TCPIP_STACK_IndexToNet(i);
                }
                appData.txrxTaskState = APP_TCPIP_WAIT_FOR_IP;
            }
        
            if (appInitialized)
            {
                I2C_Setup();
                TimerSetup();
                /* Establish the Buffer Queue event handler */
                DRV_USART_BufferEventHandlerSet(appData.handleUSART0,
                                                BufferQueueEventHandler,
                                                NULL);

                usartBQState = USART_BQ_INIT;
            
                appData.state = APP_STATE_SERVICE_TASKS;
            }
            break;
        }

        case APP_STATE_SERVICE_TASKS:
        {
		    /* Run the state machine for servicing I2C */
            APP_I2C_Task();
            USART_Task();
            TCP_Server_TXRX_Task();
        
            break;
        }

        /* TODO: implement your application state machine.*/
        
        case APP_STATE_ERROR:
        {
			break;
		}

        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}

 

/*******************************************************************************
 End of File
 */
